/******************************************************************************
 * Product: Adempiere ERP & CRM Smart Business Solution                       *
 * This program is free software; you can redistribute it and/or modify it    *
 * under the terms version 2 of the GNU General Public License as published   *
 * by the Free Software Foundation. This program is distributed in the hope   *
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.           *
 * See the GNU General Public License for more details.                       *
 * You should have received a copy of the GNU General Public License along    *
 * with this program; if not, write to the Free Software Foundation, Inc.,    *
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.                     *
 * For the text or an alternative of this public license, you may reach us    *
 * Copyright (C) 2003-2007 e-Evolution,SC. All Rights Reserved.               *
 * Contributor(s): Victor Perez www.e-evolution.com                           *
 *****************************************************************************/
package org.eevolution.process;


import java.math.*;
import java.sql.*;
import java.util.*;
import java.util.logging.*;
import org.compiere.util.*;
import org.compiere.model.Query;
import org.compiere.process.*;
import org.eevolution.model.*;


/**
 *	Component Change into BOM
 *	
 *  @author victor.perez@e-evolution.com
 *  @version $Id: ComponentChange.java
 */
public class ComponentChange extends SvrProcess
{
	/**	The Order				*/
		private int			 	p_M_Product_ID = 0;     
        private Timestamp       p_ValidTo = null;
        private Timestamp       p_ValidFrom = null;
        private String          p_Action;
        private int             p_New_M_Product_ID =0;
        private BigDecimal      p_Qty = null;
        private int 			p_M_ChangeNotice_ID=0;
        private int             morepara = 0;
	/**
	 *  Prepare - e.g., get Parameters.
	 */
	protected void prepare()
	{
		ProcessInfoParameter[] para = getParameter();
               
		for (int i = 0; i < para.length; i++)
		{
			String name = para[i].getParameterName();
                     
			if (para[i].getParameter() == null)
				;
			else if (name.equals("M_Product_ID") && morepara == 0)
            {    
				p_M_Product_ID = para[i].getParameterAsInt();
                morepara = 1;
            }
            else if (name.equals("ValidTo"))
            	p_ValidTo = ((Timestamp)para[i].getParameter());
            else if (name.equals("ValidFrom")) 
            	p_ValidFrom = ((Timestamp)para[i].getParameter());
            else if (name.equals("Action"))
            	p_Action = ((String)para[i].getParameter());
            else if (name.equals("M_Product_ID"))
            	p_New_M_Product_ID = para[i].getParameterAsInt();
            else if (name.equals("Qty")) 
            	p_Qty = ((BigDecimal)para[i].getParameter());
            else if (name.equals("M_ChangeNotice_ID")) 
    			p_M_ChangeNotice_ID = para[i].getParameterAsInt();
			else
				log.log(Level.SEVERE,"prepare - Unknown Parameter: " + name);
		}
	}	//	prepare

	/**
	 *  Perform process.
	 *  @return Message
	 *  @throws Exception if not successful
	 */
	protected String doIt() throws Exception
	{
		log.info("Existing Product" + p_M_Product_ID );
		log.info("ValidTo" + p_ValidTo);
		log.info("ValidFrom" + p_ValidFrom);
		log.info("Action" + p_Action);
		log.info("New Product" + p_New_M_Product_ID);
		log.info("Qty" + p_Qty );
		
		String whereClause = "M_Product_ID = ?";
		if (p_ValidTo !=null)
			whereClause +=" AND TRUNC(ValidTo) <= "+ DB.TO_DATE(p_ValidTo);
		if (p_ValidFrom != null)
			whereClause +=" AND TRUNC(ValidFrom) >= "+DB.TO_DATE(p_ValidFrom);
		 
		List<MPPProductBOMLine> components = new Query(getCtx(), MPPProductBOMLine.Table_Name,
				 			whereClause,get_TrxName()).setParameters(new Object[]{p_M_Product_ID}).list();
		
		for(MPPProductBOMLine bomline : components) 
		{		
            if (p_Action.equals("A"))
            {
                MPPProductBOMLine newbomline = new MPPProductBOMLine(Env.getCtx(), 0 ,get_TrxName());
                newbomline.setAssay(bomline.getAssay());
                newbomline.setBackflushGroup(bomline.getBackflushGroup());
                newbomline.setQtyBatch(bomline.getQtyBatch());
                newbomline.setC_UOM_ID(bomline.getC_UOM_ID());
                newbomline.setDescription(bomline.getDescription());
                newbomline.setHelp(bomline.getHelp());
                newbomline.setM_ChangeNotice_ID(bomline.getM_ChangeNotice_ID());  
                newbomline.setForecast(bomline.getForecast());
                newbomline.setQtyBOM(p_Qty);     
                newbomline.setComponentType(bomline.getComponentType());
                newbomline.setIsQtyPercentage(bomline.isQtyPercentage());
                newbomline.setIsCritical(bomline.isCritical());
                newbomline.setIssueMethod(bomline.getIssueMethod());
                newbomline.setLine(25);
                newbomline.setLeadTimeOffset(bomline.getLeadTimeOffset());
                newbomline.setM_AttributeSetInstance_ID(bomline.getM_AttributeSetInstance_ID());
                newbomline.setM_Product_ID(p_New_M_Product_ID);                     
                newbomline.setPP_Product_BOM_ID(bomline.getPP_Product_BOM_ID());
                newbomline.setScrap(bomline.getScrap());
                newbomline.setValidFrom(newbomline.getUpdated());
                newbomline.setM_ChangeNotice_ID(p_M_ChangeNotice_ID);
                newbomline.saveEx();
                addLog("Component add");
            }
            else if (p_Action.equals("D"))
            {
                bomline.setM_ChangeNotice_ID(p_M_ChangeNotice_ID);
                bomline.setIsActive(false);
                bomline.saveEx();
                addLog("Deactivate ");                        
            }
            else if (p_Action.equals("E"))
            {
            	bomline.setM_ChangeNotice_ID(p_M_ChangeNotice_ID);
                bomline.setValidTo(bomline.getUpdated());
                bomline.saveEx();
                addLog("Expire");                 
            }
            else  if (p_Action.equals("R"))
            {
                MPPProductBOMLine newbomline = new MPPProductBOMLine(getCtx(), 0 , get_TrxName());
                newbomline.setAssay(bomline.getAssay());
                newbomline.setBackflushGroup(bomline.getBackflushGroup());
                newbomline.setQtyBatch(bomline.getQtyBatch());
                newbomline.setComponentType(bomline.getComponentType());
                newbomline.setC_UOM_ID(bomline.getC_UOM_ID());
                newbomline.setDescription(bomline.getDescription());
                newbomline.setM_ChangeNotice_ID(bomline.getM_ChangeNotice_ID());  
                newbomline.setHelp(bomline.getHelp());
                newbomline.setForecast(bomline.getForecast());
                newbomline.setQtyBOM(p_Qty);
                newbomline.setIsQtyPercentage(bomline.isQtyPercentage());
                newbomline.setIsCritical(bomline.isCritical());
                newbomline.setIssueMethod(bomline.getIssueMethod());
                newbomline.setLine(25);
                newbomline.setLeadTimeOffset(bomline.getLeadTimeOffset());
                newbomline.setM_AttributeSetInstance_ID(bomline.getM_AttributeSetInstance_ID());
                newbomline.setM_Product_ID(p_New_M_Product_ID);                     
                newbomline.setPP_Product_BOM_ID(bomline.getPP_Product_BOM_ID());
                newbomline.setScrap(bomline.getScrap());
                newbomline.setValidFrom(newbomline.getUpdated());
                newbomline.setM_ChangeNotice_ID(p_M_ChangeNotice_ID);
                newbomline.save();
                bomline.setIsActive(false);
                bomline.setM_ChangeNotice_ID(p_M_ChangeNotice_ID);
                bomline.saveEx();
                addLog("Replace");
            }
            else if (p_Action.equals("RE"))
            {
            	MPPProductBOMLine newbomline = new MPPProductBOMLine(getCtx(), 0 , get_TrxName());
	            newbomline.setAssay(bomline.getAssay());
	            newbomline.setBackflushGroup(bomline.getBackflushGroup());
	            newbomline.setQtyBatch(bomline.getQtyBatch());
	            newbomline.setComponentType(bomline.getComponentType());
	            newbomline.setC_UOM_ID(bomline.getC_UOM_ID());
	            newbomline.setDescription(bomline.getDescription());
	            newbomline.setHelp(bomline.getHelp());
	            newbomline.setM_ChangeNotice_ID(bomline.getM_ChangeNotice_ID());                        
	            newbomline.setHelp(bomline.getHelp());
	            newbomline.setForecast(bomline.getForecast());
	            newbomline.setQtyBOM(p_Qty);
	            newbomline.setIsQtyPercentage(bomline.isQtyPercentage());
	            newbomline.setIsCritical(bomline.isCritical());
	            newbomline.setIssueMethod(bomline.getIssueMethod());
	            newbomline.setLine(25);
	            newbomline.setLeadTimeOffset(bomline.getLeadTimeOffset());
	            newbomline.setM_AttributeSetInstance_ID(bomline.getM_AttributeSetInstance_ID());
	            newbomline.setM_Product_ID(p_New_M_Product_ID);                     
	            newbomline.setPP_Product_BOM_ID(bomline.getPP_Product_BOM_ID());                     
	            newbomline.setScrap(bomline.getScrap());
	            newbomline.setValidFrom(newbomline.getUpdated());
	            newbomline.saveEx();
	            bomline.setValidTo(bomline.getUpdated());
	            bomline.setM_ChangeNotice_ID(p_M_ChangeNotice_ID);
	            bomline.saveEx();
	            addLog("Replace & Expire");
            }
            }                    
            return "@OK@";
	}	//	doIt	
}	//	Component Change
